/*
 * Created on 04/06/2008
 */
package br.pucrio.olis.business.observer;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import br.pucrio.olis.business.BusinessOperation;

/**
 * <p>
 * This class represents the default implementation of the
 * ObservableBusinessService interface.
 * </p>
 * 
 * @author ingridnunes
 * @version 1.0
 * 
 */
public class ObservableBusinessServiceImpl implements ObservableBusinessService {

	private class NotifyObserverThread extends Thread {

		private final ObservableBusinessService observable;
		private final BusinessOperation operation;

		public NotifyObserverThread(final ObservableBusinessService observable,
				final BusinessOperation operation) {
			this.observable = observable;
			this.operation = operation;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see java.lang.Thread#run()
		 */
		@Override
		public void run() {
			try {
				sleep(1000);
			} catch (Exception e) {
			}
			for (BusinessServiceObserver observer : obs) {
				observer.operationPerformed(observable, operation);
			}
		}

	}

	protected Log log;
	private List<BusinessServiceObserver> obs;

	/**
	 * <p>
	 * Creates a new instance of ObservableBusinessServiceImpl with zero
	 * observers.
	 * </p>
	 */
	public ObservableBusinessServiceImpl() {
		this.log = LogFactory.getLog(this.getClass());
		this.obs = new ArrayList<BusinessServiceObserver>();
	}

	/**
	 * <p>
	 * Adds an observer to the set of observers for this service, provided that
	 * it is not the same as some observer already in the set. The order in
	 * which notifications will be delivered to multiple observers is not
	 * specified.
	 * </p>
	 * 
	 * @param observer
	 *            an observer to be added.
	 * @throws NullPointerException
	 *             if the parameter o is null.
	 */
	public synchronized void addObserver(BusinessServiceObserver observer) {
		if (observer == null)
			throw new NullPointerException();
		if (!obs.contains(observer)) {
			obs.add(observer);
		}
	}

	/**
	 * <p>
	 * Returns the number of observers of this
	 * <tt>ObservableBusinessServiceImpl</tt> object.
	 * </p>
	 * 
	 * @return the number of observers of this observable service.
	 */
	public synchronized int countObservers() {
		return obs.size();
	}

	/**
	 * <p>
	 * Deletes an observer from the set of observers of this object. Passing
	 * <CODE>null</CODE> to this method will have no effect.
	 * </p>
	 * 
	 * @param observer
	 *            the observer to be deleted.
	 */
	public synchronized void deleteObserver(BusinessServiceObserver observer) {
		obs.remove(observer);
	}

	/**
	 * <p>
	 * Clears the observer list so that this object no longer has any observers.
	 * </p>
	 */
	public synchronized void deleteObservers() {
		obs.clear();
	}

	/**
	 * <p>
	 * Notify all of the observers that an business operation has been
	 * performed.
	 * <p>
	 * Each observer has its <code>operationPerformed</code> method called
	 * with two arguments: this observable service and the
	 * <code>operation</code> argument.
	 * </p>
	 * 
	 * @param operation
	 *            the operation performed.
	 */
	public void notifyObservers(final BusinessOperation operation) {
		log.info("An operation has been performed: " + operation);
		new NotifyObserverThread(this, operation).start();

	}

}
