/*
 * Created on May 20, 2008
 */
package br.pucrio.olis.business.service;

import java.util.Date;
import java.util.List;
import java.util.Map;

import br.pucrio.olis.business.BusinessException;
import br.pucrio.olis.business.observer.ObservableBusinessService;
import br.pucrio.olis.domain.CalendarEvent;
import br.pucrio.olis.domain.CalendarUserService;
import br.pucrio.olis.domain.Event;
import br.pucrio.olis.domain.Frequency;
import br.pucrio.olis.domain.User;

/**
 * <p>
 * This interface defines the services provided by the Calendar Service.
 * </p>
 * 
 * @author Ingrid Nunes
 * @version 1.0
 * 
 */
public interface CalendarService extends ObservableBusinessService {

	/**
	 * The String indicates the service name.
	 */
	public static final String SERVICE_NAME = CalendarService.class
			.getSimpleName();

	/**
	 * <p>
	 * Delete a calendar event from the database.
	 * </p>
	 * 
	 * @param calendarEvent
	 *            the calendar event to be delete.
	 * @throws BusinessException
	 */
	public void delete(CalendarEvent calendarEvent) throws BusinessException;

	/**
	 * <p>
	 * Retrieve a calendar event from the database according to its id.
	 * </p>
	 * 
	 * @param id
	 *            the calendar event identifier.
	 * @return the calendar event.
	 * @throws BusinessException
	 */
	public CalendarEvent getCalendarEvent(Long id) throws BusinessException;

	/**
	 * <p>
	 * Retrieve an event from the database according to its id.
	 * </p>
	 * 
	 * @param id
	 *            the event identifier.
	 * @return the event.
	 * @throws BusinessException
	 */
	public Event getEvent(Long id) throws BusinessException;

	/**
	 * <p>
	 * Load a list of calendar events. Select the calendar events according to
	 * the provided month/year. Select only the calendar events, which the
	 * provided user participates.
	 * </p>
	 * 
	 * @param month
	 *            the month that the calendar events occur.
	 * @param year
	 *            the year that the calendar events occur.
	 * @param user
	 *            the user that must participate of the calendar event.
	 * @return the map with the calendar events list for each frequency.
	 * @throws BusinessException
	 */
	public Map<Frequency, List<CalendarEvent>> loadByMonth(Integer month,
			Integer year, User user) throws BusinessException;

	/**
	 * <p>
	 * Save a calendar event in the database.
	 * </p>
	 * 
	 * @param calendarEvent
	 *            the calendar event to be saved.
	 * @return the calendar event saved.
	 * @throws BusinessException
	 */
	public CalendarEvent save(CalendarEvent calendarEvent)
			throws BusinessException;

	/**
	 * <p>
	 * Update a calendar event in the database.
	 * </p>
	 * 
	 * @param calendarEvent
	 *            the calendar event to be updated.
	 * @return the calendar event updated.
	 * @throws BusinessException
	 */
	public CalendarEvent update(CalendarEvent calendarEvent)
			throws BusinessException;

	/**
	 * <p>
	 * Update a calendar user service in the database.
	 * </p>
	 * 
	 * @param calendarUserService
	 *            the calendar service to be updated.
	 * @return the user whose calendar service was updated.
	 * @throws BusinessException
	 */
	public User update(CalendarUserService calendarUserService)
			throws BusinessException;
	
	/**
	 * <p>
	 * Load a list of calendar events. Select the calendar events according to
	 * the provided dates. Select only the calendar events, which the provided
	 * user participates.
	 * </p>
	 * 
	 * @param start
	 * @param end
	 * @param user
	 *            the user that must participate of the calendar event.
	 * @return the map with the calendar events list for each frequency.
	 * @throws BusinessException
	 */
	public Map<Frequency, List<CalendarEvent>> loadByPeriod(Date start,
			Date end, User user) throws BusinessException;

}
