/*
 * Created on May 20, 2008
 */
package br.pucrio.olis.business.service.impl;

import java.io.Serializable;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import br.pucrio.olis.business.BusinessException;
import br.pucrio.olis.business.BusinessOperation;
import br.pucrio.olis.business.BusinessOperationType;
import br.pucrio.olis.business.observer.ObservableBusinessServiceImpl;
import br.pucrio.olis.business.service.CalendarService;
import br.pucrio.olis.data.dao.CalendarEventDAO;
import br.pucrio.olis.data.dao.CalendarUserServiceDAO;
import br.pucrio.olis.data.dao.EventDAO;
import br.pucrio.olis.domain.CalendarEvent;
import br.pucrio.olis.domain.CalendarUserService;
import br.pucrio.olis.domain.Event;
import br.pucrio.olis.domain.Frequency;
import br.pucrio.olis.domain.User;
import br.pucrio.olis.util.OlisUtils;

/**
 * <p>
 * This class implements the services provided by the Calendar Service.
 * </p>
 * 
 * @author Ingrid Nunes
 * @version 1.0
 * 
 */
public class CalendarServiceImpl extends ObservableBusinessServiceImpl
		implements CalendarService {

	private CalendarEventDAO calendarEventDAO;
	private CalendarUserServiceDAO calendarUserServiceDAO;
	private EventDAO eventDAO;

	/**
	 * <p>
	 * Create an instance of CalendarServiceImpl.
	 * </p>
	 * 
	 * @param eventDAO
	 *            the event DAO.
	 * @param calendarEventDAO
	 *            the calendar event DAO.
	 */
	public CalendarServiceImpl(CalendarUserServiceDAO calendarUserServiceDAO,
			EventDAO eventDAO, CalendarEventDAO calendarEventDAO) {
		this.calendarUserServiceDAO = calendarUserServiceDAO;
		this.eventDAO = eventDAO;
		this.calendarEventDAO = calendarEventDAO;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.business.service.CalendarService#delete(br.pucrio.olis.domain.CalendarEvent)
	 */
	@Override
	public void delete(CalendarEvent calendarEvent) throws BusinessException {
		calendarEventDAO.delete(calendarEvent);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.business.service.CalendarService#getCalendarEvent(java.lang.Long)
	 */
	@Override
	public CalendarEvent getCalendarEvent(Long id) throws BusinessException {
		return calendarEventDAO.get(id);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.business.service.EventsAnnouncementService#getEvent(java.lang.Long)
	 */
	@Override
	public Event getEvent(Long id) throws BusinessException {
		return eventDAO.get(id);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.business.service.CalendarService#loadByMonth(java.lang.Integer,
	 *      java.lang.Integer, br.pucrio.olis.domain.User)
	 */
	@Override
	public Map<Frequency, List<CalendarEvent>> loadByMonth(Integer month,
			Integer year, User user) throws BusinessException {
		Date start = OlisUtils.getMonthBegin(month, year);
		Date end = OlisUtils.getMonthEnd(month, year);
		return loadByPeriod(start, end, user);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.business.service.EventSuggestionService#loadByPeriod(java.util.Date,
	 *      java.util.Date, br.pucrio.olis.domain.User)
	 */
	@Override
	public Map<Frequency, List<CalendarEvent>> loadByPeriod(Date start,
			Date end, User user) throws BusinessException {
		Map<Frequency, List<CalendarEvent>> result = new HashMap<Frequency, List<CalendarEvent>>();
		for (Frequency frequency : Frequency.values()) {
			if (Frequency.ONCE.equals(frequency)) {
				result.put(frequency, calendarUserServiceDAO
						.loadCalendarEventsBetweenDate(start, end, frequency,
								user));
			} else {
				result.put(frequency, calendarUserServiceDAO
						.loadPeriodicCalendarEventsBetweenDate(start, end,
								frequency, user));
			}
		}
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.business.service.CalendarService#save(br.pucrio.olis.domain.User,
	 *      br.pucrio.olis.domain.CalendarEvent)
	 */
	@Override
	public CalendarEvent save(CalendarEvent calendarEvent)
			throws BusinessException {
		Event event = null;
		if (calendarEvent.getEvent().getId() == null) {
			calendarEvent.getEvent().checkBusinessRules();
			Serializable id = eventDAO.save(calendarEvent.getEvent());
			event = eventDAO.get(id);
		} else {
			event = eventDAO.get(calendarEvent.getEvent().getId());
			if (event.getEditable()
					|| event.getOwner().getId().equals(
							calendarEvent.getEvent().getOwner().getId())) {
				calendarEvent.getEvent().checkBusinessRules();
				eventDAO.update(calendarEvent.getEvent());
				event = eventDAO.get(calendarEvent.getEvent().getId());
			} else {
				event = eventDAO.get(calendarEvent.getEvent().getId());
			}
		}
		calendarEvent.setEvent(event);

		Serializable id = calendarEventDAO.save(calendarEvent);

		BusinessOperation businessOperation = new BusinessOperation();
		businessOperation.setOperationType(BusinessOperationType.CREATE);
		businessOperation.setParam(calendarEventDAO.get(id));
		notifyObservers(businessOperation);

		return calendarEventDAO.get(id);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.business.service.CalendarService#update(br.pucrio.olis.domain.User,
	 *      br.pucrio.olis.domain.CalendarEvent)
	 */
	@Override
	public CalendarEvent update(CalendarEvent calendarEvent)
			throws BusinessException {
		Event event = eventDAO.get(calendarEvent.getEvent().getId());
		if (event.getEditable()
				|| event.getOwner().getId().equals(
						calendarEvent.getEvent().getOwner().getId())) {
			calendarEvent.getEvent().checkBusinessRules();
			eventDAO.update(calendarEvent.getEvent());
			event = eventDAO.get(calendarEvent.getEvent().getId());
		} else {
			event = eventDAO.get(calendarEvent.getEvent().getId());
		}
		calendarEvent.setEvent(event);

		calendarEventDAO.update(calendarEvent);
		return calendarEventDAO.get(calendarEvent.getId());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.business.service.CalendarService#update(br.pucrio.olis.domain.CalendarUserService)
	 */
	@Override
	public User update(CalendarUserService calendarUserService)
			throws BusinessException {
		calendarUserServiceDAO.update(calendarUserService);

		CalendarUserService c = calendarUserServiceDAO.get(calendarUserService
				.getId());

		BusinessOperation businessOperation = new BusinessOperation();
		businessOperation.setOperationType(BusinessOperationType.UPDATE);
		businessOperation.setParam(c);
		notifyObservers(businessOperation);

		return c.getUser();
	}
}
