/*
 * Created on Apr 4, 2008
 */
package br.pucrio.olis.business.service.impl;

import java.io.Serializable;
import java.util.Date;
import java.util.List;
import java.util.Set;

import br.pucrio.olis.business.BusinessException;
import br.pucrio.olis.business.BusinessOperation;
import br.pucrio.olis.business.BusinessOperationType;
import br.pucrio.olis.business.observer.ObservableBusinessServiceImpl;
import br.pucrio.olis.business.service.UserService;
import br.pucrio.olis.data.dao.UserDAO;
import br.pucrio.olis.domain.Notification;
import br.pucrio.olis.domain.User;
import br.pucrio.olis.util.OlisUtils;

/**
 * <p>
 * This class implements the services provided by the User Service.
 * </p>
 * 
 * @author Ingrid Nunes
 * @version 1.0
 * 
 */
@SuppressWarnings("unchecked")
public class UserServiceImpl extends ObservableBusinessServiceImpl implements
		UserService {

	private UserDAO userDAO;

	/**
	 * <p>
	 * Create an instance of UserServiceImpl.
	 * </p>
	 * 
	 * @param userDAO
	 *            the user DAO.
	 */
	public UserServiceImpl(UserDAO userDAO) {
		this.userDAO = userDAO;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.business.service.UserService#addNotification(br.pucrio.olis.domain.User,
	 *      br.pucrio.olis.domain.Notification)
	 */
	@Override
	public void addNotification(User user, Notification notification)
			throws BusinessException {
		notification.setDate(new Date());
		userDAO.addNotification(user, notification);
	}

	/**
	 * Check the user business rules.
	 * 
	 * @param user
	 *            the user to be checked.
	 * @throws BusinessException
	 */
	private void checkBusinessRules(User user) throws BusinessException {
		if (OlisUtils.isMandatoryPropertiesMissing(new String[] {
				user.getFirstName(), user.getFullName(), user.getEmail(),
				user.getUsername(), user.getPassword() })) {
			throw new BusinessException("errors.fields.required");
		}
		if (user.getDateOfBirth() == null) {
			throw new BusinessException("errors.required", "dateOfBirth");
		}
		if (user.getCity() == null) {
			throw new BusinessException("errors.required", "city");
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.business.service.UserService#deleteNotification(br.pucrio.olis.domain.User,
	 *      br.pucrio.olis.domain.Notification)
	 */
	@Override
	public void deleteNotification(User user, Notification notification)
			throws BusinessException {
		userDAO.deleteNotification(user, notification);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.business.service.UserService#loadAll()
	 */
	@Override
	public List<User> loadAll() throws BusinessException {
		return userDAO.loadAll();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.business.service.UserService#loadNotifications(br.pucrio.olis.domain.User)
	 */
	@Override
	public Set<Notification> loadNotifications(User user)
			throws BusinessException {
		return userDAO.get(user.getId()).getNotifications();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.business.service.UserService#loadUserById(java.lang.Long)
	 */
	@Override
	public User loadUserById(Long id) throws BusinessException {
		return userDAO.get(id);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.business.service.UserService#loadUsers()
	 */
	@Override
	public List<User> loadUsers() throws BusinessException {
		return userDAO.loadAll();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.business.service.UserService#login(java.lang.String,
	 *      java.lang.String)
	 */
	@Override
	public User login(String username, String password)
			throws BusinessException {
		User user = userDAO.loadUserByUsername(username);

		if ((user != null) && !user.getPassword().equals(password)) {
			user = null;
		}
		if (user == null) {
			throw new BusinessException("error.password.mismatch");
		}

		return user;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.business.service.UserService#save(br.pucrio.olis.domain.User)
	 */
	@Override
	public User save(User user) throws BusinessException {
		User u = userDAO.loadUserByUsername(user.getUsername());
		if (u != null) {
			throw new BusinessException("error.username.unique");
		}
		checkBusinessRules(user);
		Serializable id = userDAO.save(user);

		BusinessOperation businessOperation = new BusinessOperation();
		businessOperation.setOperationType(BusinessOperationType.CREATE);
		businessOperation.setParam(userDAO.get(id));
		notifyObservers(businessOperation);

		return userDAO.get(id);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.business.service.UserService#update(br.pucrio.olis.domain.User)
	 */
	@Override
	public User update(User user) throws BusinessException {
		User u = userDAO.loadUserByUsername(user.getUsername());
		u.setFirstName(user.getFirstName());
		u.setLastName(user.getLastName());
		u.setPassword(user.getPassword());
		u.setEmail(user.getEmail());
		u.setDateOfBirth(user.getDateOfBirth());
		u.setCity(user.getCity());
		userDAO.update(u);
		return userDAO.get(u.getId());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.business.service.UserService#updateAdvancedOptions(br.pucrio.olis.domain.User)
	 */
	@Override
	public User updateAdvancedOptions(User user) throws BusinessException {
		userDAO.updateAdvancedOptions(user);
		return userDAO.get(user.getId());
	}

}
