/*
 * Created on May 1, 2008
 */
package br.pucrio.olis.domain;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import javax.persistence.CascadeType;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.FetchType;
import javax.persistence.ManyToMany;
import javax.persistence.OneToMany;
import javax.persistence.Transient;

import br.pucrio.olis.business.BusinessException;

/**
 * <p>
 * This class represents an academic event, which has some academic specific
 * properties, besides the common properties of the Event class.
 * </p>
 * 
 * @author Ingrid Nunes
 * @version 1.0
 * 
 */
@Entity
public class AcademicEvent extends Event {

	/**
	 * <p>
	 * UID generated.
	 * </p>
	 */
	private static final long serialVersionUID = 2891168449731718885L;

	/**
	 * The Set represents the set of areas of interest related to this academic
	 * event.
	 */
	private Set<AreaOfInterest> areasOfInterest;
	/**
	 * The List represents the deadlines of this academic event.
	 */
	private List<DeadlineDate> deadlines;
	/**
	 * The AcademicEventType represent the type of the academic event of this
	 * academic event.
	 */
	private AcademicEventType eventType;

	/**
	 * <p>
	 * Create an instance of AcademicEvent.
	 * </p>
	 */
	public AcademicEvent() {
		this.areasOfInterest = new HashSet<AreaOfInterest>();
		this.deadlines = new ArrayList<DeadlineDate>();
		for (Deadline deadline : Deadline.values()) {
			putDeadline(deadline, null);
		}
	}

	/**
	 * <p>
	 * Add an area of interest.
	 * </p>
	 * 
	 * @param areaOfInterest
	 *            the area of interest to be added.
	 */
	public void addAreasOfInterest(AreaOfInterest areaOfInterest) {
		this.areasOfInterest.add(areaOfInterest);
	}

	/**
	 * <p>
	 * Add a deadline date.
	 * </p>
	 * 
	 * @param deadline
	 * @param date
	 */
	public void addDeadline(Deadline deadline, Date date) {
		this.deadlines.add(new DeadlineDate(deadline, date, this));
	}

	/**
	 * <p>
	 * Add a deadline date.
	 * </p>
	 * 
	 * @param deadlineDate
	 */
	public void addDeadlines(DeadlineDate deadlineDate) {
		Iterator<DeadlineDate> it = deadlines.iterator();
		while (it.hasNext()) {
			DeadlineDate dd = it.next();
			if (dd.getDeadline().equals(deadlineDate.getDeadline())) {
				it.remove();
				break;
			}
		}
		this.deadlines.add(deadlineDate);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.domain.Event#checkBusinessRules()
	 */
	@Override
	public void checkBusinessRules() throws BusinessException {
		super.checkBusinessRules();
		if (eventType == null) {
			throw new BusinessException("errors.required", "eventType");
		}
	}

	@Transient
	public Iterator<AreaOfInterest> getAllAreasOfInterest() {
		return areasOfInterest.iterator();
	}

	@Transient
	public Iterator<DeadlineDate> getAllDeadlines() {
		return deadlines.iterator();
	}

	@ManyToMany(fetch = FetchType.EAGER)
	public Set<AreaOfInterest> getAreasOfInterest() {
		return areasOfInterest;
	}

	/**
	 * <p>
	 * Retrieve the date of a specific type of deadline.
	 * </p>
	 * 
	 * @param deadline
	 *            the deadline type.
	 * @return the deadline date.
	 */
	@Transient
	public Date getDeadline(Deadline deadline) {
		for (DeadlineDate deadlineDate : deadlines) {
			if (deadlineDate.getDeadline().equals(deadline))
				return deadlineDate.getDate();
		}
		return null;
	}

	@OneToMany(mappedBy = "academicEvent", cascade = CascadeType.ALL, fetch = FetchType.EAGER)
	public List<DeadlineDate> getDeadlines() {
		return deadlines;
	}

	@Enumerated(EnumType.STRING)
	public AcademicEventType getEventType() {
		return eventType;
	}

	/**
	 * <p>
	 * Set the date for a specific deadline type.
	 * </p>
	 * 
	 * @param deadline
	 *            the deadline type.
	 * @param date
	 *            the deadline date.
	 */
	public void putDeadline(Deadline deadline, Date date) {
		for (DeadlineDate deadlineDate : deadlines) {
			if (deadlineDate.getDeadline().equals(deadline)) {
				deadlineDate.setDate(date);
				return;
			}
		}
		addDeadline(deadline, date);
	}

	/**
	 * <p>
	 * Remove an area of interest.
	 * </p>
	 * 
	 * @param areaOfInterest
	 *            the area of interest to be removed.
	 */
	public boolean removeAreaOfInterests(AreaOfInterest areaOfInterest) {
		return this.areasOfInterest.remove(areaOfInterest);
	}

	public void setAreasOfInterest(Set<AreaOfInterest> areasOfInterest) {
		this.areasOfInterest = areasOfInterest;
	}

	public void setDeadlines(List<DeadlineDate> deadlines) {
		this.deadlines = deadlines;
	}

	public void setEventType(AcademicEventType eventType) {
		this.eventType = eventType;
	}

}
