/*
 * Created on May 1, 2008
 */
package br.pucrio.olis.domain;

import jade.content.Concept;

import java.io.Serializable;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Set;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Inheritance;
import javax.persistence.InheritanceType;
import javax.persistence.ManyToOne;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Transient;

import org.hibernate.annotations.Type;

import br.pucrio.olis.business.BusinessException;
import br.pucrio.olis.util.OlisUtils;

/**
 * <p>
 * This class represents a generic event. Events can be announced or be part of
 * a calendar event.
 * </p>
 * 
 * @author Ingrid Nunes
 * @version 1.0
 * 
 */
@Entity
@Inheritance(strategy = InheritanceType.JOINED)
public class Event implements Serializable, Concept {

	/**
	 * <p>
	 * UID generated.
	 * </p>
	 */
	private static final long serialVersionUID = -4974210532887219920L;

	/**
	 * The Boolean represents if the event is an all day event.
	 */
	protected Boolean allDay;
	/**
	 * The City represents on which city the event is going to happen.
	 */
	protected City city;
	/**
	 * The String represents the description of the event.
	 */
	protected String description;
	/**
	 * The Date represents when the event ends.
	 */
	protected Date end;
	/**
	 * The Date represents the time that the event ends.
	 */
	protected Date endTime;
	/**
	 * The EventsBoard represents the events board that this event makes part
	 * of.
	 */
	protected EventsBoard eventsBoard;
	/**
	 * The Frequency represents on which frequency the event occurs.
	 */
	private Frequency frequency;
	/**
	 * The Long represents the event unique identifier.
	 */
	protected Long id;
	/**
	 * The String represents the location of the event.
	 */
	protected String location;
	/**
	 * The User represents the user who created the event.
	 */
	protected User owner;
	/**
	 * The Date represents when the event starts.
	 */
	protected Date start;
	/**
	 * The Date represents the time that the event starts.
	 */
	protected Date startTime;
	/**
	 * The String represents the subject of the event.
	 */
	protected String subject;

	/**
	 * <p>
	 * Create an instance of Event.
	 * </p>
	 */
	public Event() {
		this.frequency = Frequency.ONCE;
	}

	/**
	 * Check the event business rules.
	 * 
	 * @throws BusinessException
	 */
	public void checkBusinessRules() throws BusinessException {
		if (OlisUtils.isEmpty(subject)) {
			throw new BusinessException("error.required", "subject");
		}
		if (city == null) {
			throw new BusinessException("error.required", "city");
		}
		if (start == null) {
			throw new BusinessException("error.required", "start");
		}
		if (end == null) {
			throw new BusinessException("error.required", "end");
		}
		if (frequency == null) {
			throw new BusinessException("error.required", "frequency");
		}
		if (start.compareTo(end) > 0) {
			throw new BusinessException("error.start.before.end");
		}
		if (Frequency.ONCE.equals(frequency)) {
			if (startTime != null) {
				throw new BusinessException("error.blank", "startTime");
			}
			if (endTime != null) {
				throw new BusinessException("error.blank", "endTime");
			}
		} else {
			if (startTime == null) {
				throw new BusinessException("error.required", "startTime");
			}
			if (endTime == null) {
				throw new BusinessException("error.required", "endTime");
			}
			if (startTime.compareTo(endTime) > 0) {
				throw new BusinessException("error.start.before.end");
			}
		}
		if (owner == null) {
			throw new BusinessException("errors.required", "owner");
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (!(obj instanceof Event))
			return false;
		return id.equals(((Event) obj).getId());
	}

	@Type(type = "yes_no")
	public Boolean getAllDay() {
		return allDay;
	}

	@ManyToOne
	public City getCity() {
		return city;
	}

	public String getDescription() {
		return description;
	}

	/**
	 * <p>
	 * Check if the event is editable. An event is not editable when it makes
	 * part of the events board, which means that it is an announced event.
	 * </p>
	 * 
	 * @return true is the event is editable.
	 */
	@Transient
	public Boolean getEditable() {
		return (eventsBoard == null);
	}

	@Temporal(TemporalType.TIMESTAMP)
	@Column(nullable = false)
	public Date getEnd() {
		return end;
	}

	@Temporal(TemporalType.TIME)
	public Date getEndTime() {
		return endTime;
	}

	@ManyToOne
	public EventsBoard getEventsBoard() {
		return eventsBoard;
	}

	@Enumerated(EnumType.STRING)
	public Frequency getFrequency() {
		return frequency;
	}

	@Id
	@GeneratedValue(strategy = GenerationType.AUTO)
	public Long getId() {
		return id;
	}

	public String getLocation() {
		return location;
	}

	/**
	 * <p>
	 * Calculates the next time that the event will occur.
	 * </p>
	 * 
	 * @return
	 */
	@Transient
	public Date getNextTime() {
		return getNextTime(new Date());
	}

	@Transient
	public Date getNextTime(Date now) {
		if (hasExpired(now))
			return null;
		if (Frequency.ONCE.equals(frequency)) {
			return start;
		} else {
			Calendar calendar = GregorianCalendar.getInstance();
			calendar.setTime(now);
			if (!allDay) {
				Calendar time = GregorianCalendar.getInstance();
				time.setTime(startTime);
				calendar.set(Calendar.HOUR_OF_DAY, time
						.get(Calendar.HOUR_OF_DAY));
				calendar.set(Calendar.MINUTE, time.get(Calendar.MINUTE));
			} else {
				calendar.set(Calendar.HOUR_OF_DAY, 0);
				calendar.set(Calendar.MINUTE, 0);
			}
			calendar.set(Calendar.SECOND, 0);
			calendar.set(Calendar.MILLISECOND, 0);

			if (now.after(calendar.getTime())) {
				calendar.add(Calendar.DAY_OF_MONTH, 1);
			}

			Set<Integer> validDays = Frequency.DAYS.get(frequency);
			while (!validDays.contains(calendar.get(Calendar.DAY_OF_WEEK))) {
				calendar.add(Calendar.DAY_OF_MONTH, 1);
			}

			return calendar.getTime();
		}
	}

	@ManyToOne
	public User getOwner() {
		return owner;
	}

	@Transient
	public Date getPreviousTime(Date now) {
		if (inTheFuture(now))
			return null;
		if (Frequency.ONCE.equals(frequency)) {
			return end;
		} else {
			Calendar calendar = GregorianCalendar.getInstance();
			calendar.setTime(now);
			if (!allDay) {
				Calendar time = GregorianCalendar.getInstance();
				time.setTime(startTime);
				calendar.set(Calendar.HOUR_OF_DAY, time
						.get(Calendar.HOUR_OF_DAY));
				calendar.set(Calendar.MINUTE, time.get(Calendar.MINUTE));
			} else {
				calendar.set(Calendar.HOUR_OF_DAY, 0);
				calendar.set(Calendar.MINUTE, 0);
			}
			calendar.set(Calendar.SECOND, 0);
			calendar.set(Calendar.MILLISECOND, 0);

			if (now.before(calendar.getTime())) {
				calendar.add(Calendar.DAY_OF_MONTH, -1);
			}

			Set<Integer> validDays = Frequency.DAYS.get(frequency);
			while (!validDays.contains(calendar.get(Calendar.DAY_OF_WEEK))) {
				calendar.add(Calendar.DAY_OF_MONTH, -1);
			}

			if (!allDay) {
				Calendar time = GregorianCalendar.getInstance();
				time.setTime(endTime);
				calendar.set(Calendar.HOUR_OF_DAY, time
						.get(Calendar.HOUR_OF_DAY));
				calendar.set(Calendar.MINUTE, time.get(Calendar.MINUTE));
			} else {
				calendar.set(Calendar.HOUR_OF_DAY, 23);
				calendar.set(Calendar.MINUTE, 59);
			}

			return calendar.getTime();
		}
	}

	@Temporal(TemporalType.TIMESTAMP)
	@Column(nullable = false)
	public Date getStart() {
		return start;
	}

	@Temporal(TemporalType.TIME)
	public Date getStartTime() {
		return startTime;
	}

	@Column(nullable = false)
	public String getSubject() {
		return subject;
	}

	/**
	 * <p>
	 * Checks if the event has already expired.
	 * </p>
	 * 
	 * @return true if the event has expired.
	 */
	@Transient
	public boolean hasExpired() {
		return hasExpired(new Date());
	}

	/**
	 * <p>
	 * Checks if the event has already expired.
	 * </p>
	 * 
	 * @return true if the event has expired.
	 */
	@Transient
	public boolean hasExpired(Date now) {
		if (Frequency.ONCE.equals(frequency)) {
			return now.after(start);
		} else {
			Calendar calendar = GregorianCalendar.getInstance();
			calendar.setTime(end);
			if (!allDay) {
				Calendar time = GregorianCalendar.getInstance();
				time.setTime(startTime);
				calendar.set(Calendar.HOUR_OF_DAY, time
						.get(Calendar.HOUR_OF_DAY));
				calendar.set(Calendar.MINUTE, time.get(Calendar.MINUTE));
			} else {
				calendar.set(Calendar.HOUR_OF_DAY, 0);
				calendar.set(Calendar.MINUTE, 0);
			}
			calendar.set(Calendar.SECOND, 0);
			calendar.set(Calendar.MILLISECOND, 0);

			Set<Integer> validDays = Frequency.DAYS.get(frequency);
			while (!validDays.contains(calendar.get(Calendar.DAY_OF_WEEK))) {
				calendar.add(Calendar.DAY_OF_MONTH, 1);
			}

			return now.after(calendar.getTime());
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		return id.hashCode();
	}

	@Transient
	public boolean inTheFuture(Date now) {
		if (Frequency.ONCE.equals(frequency)) {
			return start.after(now);
		} else {
			Calendar calendar = GregorianCalendar.getInstance();
			calendar.setTime(start);
			if (!allDay) {
				Calendar time = GregorianCalendar.getInstance();
				time.setTime(endTime);
				calendar.set(Calendar.HOUR_OF_DAY, time
						.get(Calendar.HOUR_OF_DAY));
				calendar.set(Calendar.MINUTE, time.get(Calendar.MINUTE));
			} else {
				calendar.set(Calendar.HOUR_OF_DAY, 0);
				calendar.set(Calendar.MINUTE, 0);
			}

			Set<Integer> validDays = Frequency.DAYS.get(frequency);
			while (!validDays.contains(calendar.get(Calendar.DAY_OF_WEEK))) {
				calendar.add(Calendar.DAY_OF_MONTH, 1);
			}

			return calendar.getTime().after(now);
		}
	}

	public void setAllDay(Boolean allDay) {
		this.allDay = allDay;
	}

	public void setCity(City city) {
		this.city = city;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public void setEnd(Date end) {
		this.end = end;
	}

	public void setEndTime(Date endTime) {
		this.endTime = endTime;
	}

	public void setEventsBoard(EventsBoard eventsBoard) {
		this.eventsBoard = eventsBoard;
	}

	public void setFrequency(Frequency frequency) {
		this.frequency = frequency;
	}

	public void setId(Long id) {
		this.id = id;
	}

	public void setLocation(String location) {
		this.location = location;
	}

	public void setOwner(User owner) {
		this.owner = owner;
	}

	public void setStart(Date start) {
		this.start = start;
	}

	public void setStartTime(Date startTime) {
		this.startTime = startTime;
	}

	public void setSubject(String subject) {
		this.subject = subject;
	}

}
