/*
 * Created on Apr 4, 2008
 */
package br.pucrio.olis.domain;

import jade.content.Concept;

import java.io.Serializable;
import java.util.Date;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;
import javax.persistence.OrderBy;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Transient;

/**
 * <p>
 * This class represents a user from the system.
 * </p>
 * 
 * @author Ingrid Nunes
 * @version 1.0
 * 
 */
@Entity
public class User implements Serializable, Concept {

	/**
	 * <p>
	 * UID generated.
	 * </p>
	 */
	private static final long serialVersionUID = -5355565940585265536L;

	/**
	 * The City represents the city where the user lives.
	 */
	private City city;
	/**
	 * The DateFormat represents the format that the dates should be displayed
	 * in the system.
	 */
	private DateFormat dateFormat;
	/**
	 * The Date represents the user date of birth.
	 */
	private Date dateOfBirth;
	/**
	 * The String represents the e-mail of the user.
	 */
	private String email;
	/**
	 * The String represents the first name of the user.
	 */
	private String firstName;
	/**
	 * The Long represents the user unique identifier.
	 */
	private Long id;
	/**
	 * The String represents the last name of the user.
	 */
	private String lastName;
	/**
	 * The Set represents notifications for the user.
	 */
	private Set<Notification> notifications;
	/**
	 * The String represents the password of the user.
	 */
	private String password;
	/**
	 * The UserPreferences represents the preferences of the user.
	 */
	private UserPreferences preferences;
	/**
	 * The Set represents the set of user services.
	 */
	private Set<UserService> services;
	/**
	 * The TimeFormat represents the format that the times should be displayed
	 * in the system.
	 */
	private TimeFormat timeFormat;
	/**
	 * The String represents the user name.
	 */
	private String username;

	/**
	 * <p>
	 * Create a new instance of User.
	 * </p>
	 */
	public User() {
		this.dateFormat = DateFormat.DAY_MONTH_YEAR;
		this.timeFormat = TimeFormat.HOURS_24;
		this.services = new HashSet<UserService>();
		this.notifications = new HashSet<Notification>();
	}

	public void addNotifications(Notification notification) {
		this.notifications.add(notification);
	}
	
	/**
	 * <p>
	 * Add a user service to this user.
	 * </p>
	 * 
	 * @param service
	 *            the service to be added.
	 */
	public void addServices(UserService service) {
		service.setUser(this);
		this.services.add(service);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (!(obj instanceof User))
			return false;
		return id.equals(((User) obj).getId());
	}
	
	@Transient
	public Iterator<Notification> getAllNotifications() {
		return notifications.iterator();
	}

	@Transient
	public Iterator<UserService> getAllServices() {
		return services.iterator();
	}

	@ManyToOne
	public City getCity() {
		return city;
	}

	@Enumerated(EnumType.STRING)
	public DateFormat getDateFormat() {
		return dateFormat;
	}

	@Temporal(TemporalType.DATE)
	public Date getDateOfBirth() {
		return dateOfBirth;
	}

	@Column(nullable = false)
	public String getEmail() {
		return email;
	}

	@Column(nullable = false)
	public String getFirstName() {
		return firstName;
	}

	/**
	 * <p>
	 * Return the full name of the user. It is the user first name, followed by
	 * the user last name.
	 * </p>
	 * 
	 * @return the user full name.
	 */
	@Transient
	public String getFullName() {
		return new StringBuffer(firstName).append(" ").append(lastName)
				.toString();
	}

	@Id
	@GeneratedValue(strategy = GenerationType.AUTO)
	public Long getId() {
		return id;
	}

	@Column(nullable = false)
	public String getLastName() {
		return lastName;
	}

	@OneToMany(fetch = FetchType.EAGER)
	@OrderBy("date")
	public Set<Notification> getNotifications() {
		return notifications;
	}

	@Column(nullable = false, length = 50)
	public String getPassword() {
		return password;
	}

	@OneToOne(mappedBy = "user")
	public UserPreferences getPreferences() {
		return preferences;
	}

	/**
	 * <p>
	 * Return an user service of a specific class.
	 * </p>
	 * 
	 * @param serviceClass
	 *            the service class.
	 * @return the service or null if no service was found.
	 */
	@SuppressWarnings("unchecked")
	@Transient
	public UserService getService(Class serviceClass) {
		for (UserService userService : services) {
			if (userService.getClass().equals(serviceClass)) {
				return userService;
			}
		}
		return null;
	}

	@OneToMany(mappedBy = "user", fetch = FetchType.EAGER)
	@OrderBy("id")
	public Set<UserService> getServices() {
		return services;
	}

	@Enumerated(EnumType.STRING)
	public TimeFormat getTimeFormat() {
		return timeFormat;
	}

	@Column(unique = true, nullable = false, length = 50)
	public String getUsername() {
		return username;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		return id.hashCode();
	}

	/**
	 * <p>
	 * Remove a user service from this user.
	 * </p>
	 * 
	 * @param service
	 *            the service to be removed.
	 */
	public boolean removeServices(UserService service) {
		service.setUser(null);
		return this.services.remove(service);
	}

	@Column(nullable = false)
	public void setCity(City city) {
		this.city = city;
	}

	public void setDateFormat(DateFormat dateFormat) {
		this.dateFormat = dateFormat;
	}

	public void setDateOfBirth(Date dateOfBirth) {
		this.dateOfBirth = dateOfBirth;
	}

	public void setEmail(String email) {
		this.email = email;
	}

	public void setFirstName(String firstName) {
		this.firstName = firstName;
	}

	public void setId(Long id) {
		this.id = id;
	}

	public void setLastName(String lastName) {
		this.lastName = lastName;
	}

	public void setNotifications(Set<Notification> notifications) {
		this.notifications = notifications;
	}

	public void setPassword(String password) {
		this.password = password;
	}

	public void setPreferences(UserPreferences preferences) {
		if (this.preferences != null) {
			this.preferences.setUser(null);
		}
		if (preferences != null) {
			preferences.setUser(this);
		}
		this.preferences = preferences;
	}

	public void setServices(Set<UserService> services) {
		this.services = services;
	}
	
	public void setTimeFormat(TimeFormat timeFormat) {
		this.timeFormat = timeFormat;
	}

	public void setUsername(String username) {
		this.username = username;
	}

}
