/*
 * Created on May 24, 2008
 */
package br.pucrio.olis.web;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import br.pucrio.olis.business.service.AcademicEventService;
import br.pucrio.olis.business.service.LocationService;
import br.pucrio.olis.domain.AcademicEvent;
import br.pucrio.olis.domain.AcademicEventType;
import br.pucrio.olis.domain.AreaOfInterest;
import br.pucrio.olis.domain.Deadline;
import br.pucrio.olis.domain.Event;

/**
 * <p>
 * This class represents the data manager of academic events. It contains the
 * fields and lists necessary to manage academic events. It is used by different
 * actions.
 * </p>
 * 
 * @author Ingrid Nunes
 * @version 1.0
 * 
 */
public class AcademicEventDataManager extends EventDataManager {

	/**
	 * <p>
	 * UID generated.
	 * </p>
	 */
	private static final long serialVersionUID = -9011475510058970442L;

	private AcademicEventService academicEventService;
	private List<AreaOfInterest> areaOfInterestList;
	private List<Long> areasOfInterest;
	private List<String> deadlines;
	private String eventType;

	/**
	 * <p>
	 * Create an instance of AcademicEventAbstractAction.
	 * </p>
	 * 
	 * @param academicEventService
	 *            the academic event service.
	 * @param locationService
	 *            the location service.
	 */
	protected AcademicEventDataManager(
			AcademicEventService academicEventService,
			LocationService locationService) {
		super(locationService);
		this.academicEventService = academicEventService;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.web.EventAbstractAction#createEvent()
	 */
	@Override
	public Event createEvent() throws Exception {
		return new AcademicEvent();
	}

	public List<AcademicEventType> getAcademicEventTypes() {
		return Arrays.asList(AcademicEventType.values());
	}

	public List<AreaOfInterest> getAreaOfInterestList() {
		return areaOfInterestList;
	}

	public List<Long> getAreasOfInterest() {
		return areasOfInterest;
	}

	public List<Deadline> getDeadlineList() {
		return Arrays.asList(Deadline.values());
	}

	public List<String> getDeadlines() {
		return deadlines;
	}

	/* (non-Javadoc)
	 * @see br.pucrio.olis.web.EventDataManager#getEventPage()
	 */
	@Override
	public String getEventPage() {
		return "AcademicEventInput";
	}

	public String getEventType() {
		return eventType;
	}
	
	public List<AcademicEventType> getEventTypes() {
		return Arrays.asList(AcademicEventType.values());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.web.EventAbstractAction#initializeFields(br.pucrio.olis.domain.Event)
	 */
	@Override
	public void initializeFields(Event event) throws Exception {
		super.initializeFields(event);
		if (event == null) {
			List<Deadline> deadlineList = getDeadlineList();
			this.deadlines = new ArrayList<String>(deadlineList.size());
			for (int i = 0; i < deadlineList.size(); i++) {
				this.deadlines.add(null);
			}
		} else {
			AcademicEvent academicEvent = (AcademicEvent) event;
			this.areasOfInterest = new ArrayList<Long>(academicEvent
					.getAreasOfInterest().size());
			for (AreaOfInterest areaOfInterest : academicEvent
					.getAreasOfInterest()) {
				this.areasOfInterest.add(areaOfInterest.getId());
			}

			SimpleDateFormat dateFormat = new SimpleDateFormat(
					olisActionSupport.getUser().getDateFormat().getFormat());
			List<Deadline> deadlineList = getDeadlineList();
			this.deadlines = new ArrayList<String>(deadlineList.size());
			for (Deadline deadline : deadlineList) {
				Date deadlineDate = academicEvent.getDeadline(deadline);
				if (deadlineDate == null) {
					this.deadlines.add(null);
				} else {
					this.deadlines.add(dateFormat.format(deadlineDate));
				}
			}

			this.eventType = academicEvent.getEventType().toString();
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.web.EventDataManager#prepare()
	 */
	@Override
	public void prepare() throws Exception {
		super.prepare();
		this.areaOfInterestList = academicEventService.loadAllAreasOfInterest();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.web.EventAbstractAction#readEvent()
	 */
	@Override
	public Event readEvent() throws Exception {
		AcademicEvent event = (AcademicEvent) super.readEvent();

		if (areasOfInterest != null) {
			for (Long id : areasOfInterest) {
				AreaOfInterest areaOfInterest = new AreaOfInterest();
				areaOfInterest.setId(id);
				event.addAreasOfInterest(areaOfInterest);
			}
		}

		SimpleDateFormat dateFormat = new SimpleDateFormat(olisActionSupport
				.getUser().getDateFormat().getFormat());
		Iterator<String> deadlineIt = deadlines.iterator();
		for (Deadline deadline : getDeadlineList()) {
			String deadlineDate = deadlineIt.next();
			if (deadlineDate != null && !"".equals(deadlineDate)) {
				try {
					event.putDeadline(deadline, dateFormat.parse(deadlineDate));
				} catch (Exception e) {
					olisActionSupport.addActionError(olisActionSupport
							.getText("error.deadline"));
				}
			} else {
				event.putDeadline(deadline, null);
			}
		}

		try {
			event.setEventType(AcademicEventType.valueOf(eventType));
		} catch (Exception exc) {
			olisActionSupport.addFieldError("eventDataManager.eventType",
					"error.invalid", "eventType");
		}
		return event;
	}

	public void setAreaOfInterestList(List<AreaOfInterest> areaOfInterestList) {
		this.areaOfInterestList = areaOfInterestList;
	}

	public void setAreasOfInterest(List<Long> areasOfInterest) {
		this.areasOfInterest = areasOfInterest;
	}

	public void setDeadlines(List<String> deadlines) {
		this.deadlines = deadlines;
	}

	public void setEventType(String eventType) {
		this.eventType = eventType;
	}

}
