/*
 * Created on May 24, 2008
 */
package br.pucrio.olis.web;

import java.io.Serializable;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import br.pucrio.olis.business.BusinessException;
import br.pucrio.olis.business.service.LocationService;
import br.pucrio.olis.domain.City;
import br.pucrio.olis.domain.Country;
import br.pucrio.olis.domain.DateFormat;
import br.pucrio.olis.domain.Event;
import br.pucrio.olis.domain.EventsBoard;
import br.pucrio.olis.domain.Frequency;
import br.pucrio.olis.domain.TimeFormat;
import br.pucrio.olis.domain.User;

/**
 * <p>
 * This class represents the data manager of events. It contains the fields and
 * lists necessary to manage events. It is used by different actions.
 * </p>
 * 
 * @author Ingrid Nunes
 * @version 1.0
 * 
 */
public class EventDataManager implements Serializable {

	/**
	 * <p>
	 * UID generated.
	 * </p>
	 */
	private static final long serialVersionUID = -9066442261373110004L;

	protected Boolean allDay;
	protected City city;
	protected List<Country> countries;
	protected Country country;
	protected String description;
	protected Boolean editable;
	protected String end;
	protected String endTime;
	protected Long eventId;
	protected String frequency;
	protected String location;
	protected LocationService locationService;
	protected Integer month;
	protected OlisActionSupport olisActionSupport;
	protected String start;
	protected String startTime;
	protected String subject;
	protected Integer year;

	/**
	 * <p>
	 * Create an instance of EventDataManager.
	 * </p>
	 * 
	 * @param olisActionSupport
	 *            the olis action support.
	 * @param locationService
	 *            the location service.
	 */
	protected EventDataManager(LocationService locationService) {
		this.locationService = locationService;
	}

	/**
	 * <p>
	 * Create an instance of an Event.
	 * </p>
	 * 
	 * @return the event.
	 * @throws Exception
	 */
	public Event createEvent() throws Exception {
		return new Event();
	}

	public Boolean getAllDay() {
		return allDay;
	}

	public List<City> getCities() throws BusinessException {
		if (country == null) {
			return new ArrayList<City>();
		} else {
			return locationService.loadCitiesByCountry(country);
		}
	}

	public City getCity() {
		return city;
	}

	public List<Country> getCountries() {
		return countries;
	}

	public Country getCountry() {
		return country;
	}

	public String getDescription() {
		return description;
	}

	public Boolean getEditable() {
		return editable;
	}

	public String getEnd() {
		return end;
	}

	public String getEndTime() {
		return endTime;
	}

	public Long getEventId() {
		return eventId;
	}

	public String getEventPage() {
		return null;
	}

	public List<Frequency> getFrequencies() {
		return Arrays.asList(Frequency.values());
	}

	public String getFrequency() {
		return frequency;
	}

	public String getLocation() {
		return location;
	}

	public Integer getMonth() {
		return month;
	}
	
	public OlisActionSupport getOlisActionSupport() {
		return olisActionSupport;
	}

	public String getStart() {
		return start;
	}

	public String getStartTime() {
		return startTime;
	}

	public String getSubject() {
		return subject;
	}

	public Integer getYear() {
		return year;
	}

	/**
	 * <p>
	 * Initializes the action fields according to a event.
	 * </p>
	 * 
	 * @param event
	 *            the event.
	 * @throws Exception
	 */
	public void initializeFields(Event event) throws Exception {
		User user = olisActionSupport.getUser();
		if (event == null) {
			this.editable = true;
			this.city = user.getCity();
			this.country = user.getCity().getCountry();
		} else {
			allDay = event.getAllDay();
			city = event.getCity();
			country = event.getCity().getCountry();
			description = event.getDescription();
			editable = event.getEditable()
					|| (user != null && user.equals(event.getOwner()));
			eventId = event.getId();
			frequency = event.getFrequency().toString();
			location = event.getLocation();
			subject = event.getSubject();
			SimpleDateFormat dateFormat = new SimpleDateFormat(
					DateFormat.DAY_MONTH_YEAR.getFormat());
			SimpleDateFormat timeFormat = new SimpleDateFormat(
					TimeFormat.HOURS_24.getFormat());
			if (user != null) {
				dateFormat = new SimpleDateFormat(user.getDateFormat()
						.getFormat());
				timeFormat = new SimpleDateFormat(user.getTimeFormat()
						.getFormat());
			}
			start = dateFormat.format(event.getStart());
			end = dateFormat.format(event.getEnd());
			if (event.getAllDay()) {
				startTime = null;
				endTime = null;
			} else if (Frequency.ONCE.equals(event.getFrequency())) {
				startTime = timeFormat.format(event.getStart());
				endTime = timeFormat.format(event.getEnd());
			} else {
				startTime = timeFormat.format(event.getStartTime());
				endTime = timeFormat.format(event.getEndTime());
			}
		}
	}

	/**
	 * <p>
	 * Prepare the necessary fields of the data manager.
	 * </p>
	 * 
	 * @throws Exception
	 */
	public void prepare() throws Exception {
		this.countries = locationService.loadAllCountries();
	}

	/**
	 * <p>
	 * Return the event with the fields content.
	 * </p>
	 */
	public Event readEvent() throws Exception {
		Event event = createEvent();
		if (editable) {
			User user = olisActionSupport.getUser();
			event.setId(eventId);
			event.setAllDay(allDay);
			event.setSubject(subject);
			event.setDescription(description);
			event.setLocation(location);
			event.setCity(city);
			try {
				event.setFrequency(Frequency.valueOf(frequency));
			} catch (Exception e) {
				olisActionSupport.addFieldError("eventDataManager.frequency",
						"error.invalid", "frequency");
			}
			event.setOwner(user);

			String startTime = this.startTime;
			String endTime = this.endTime;
			if (event.getAllDay()) {
				startTime = "00:00";
				endTime = "23:59";
			}

			if (Frequency.ONCE.equals(event.getFrequency())) {
				SimpleDateFormat dateTimeFormat = new SimpleDateFormat(user
						.getDateFormat().getFormat()
						+ " " + user.getTimeFormat().getFormat());
				try {
					event.setStart(dateTimeFormat
							.parse(start + " " + startTime));
				} catch (Exception e) {
					olisActionSupport.addFieldError("eventDataManager.start",
							"error.invalid", "start");
				}
				try {
					event.setEnd(dateTimeFormat.parse(end + " " + endTime));
				} catch (Exception e) {
					olisActionSupport.addFieldError("eventDataManager.end",
							"error.invalid", "end");
				}
				event.setStartTime(null);
				event.setEndTime(null);
			} else {
				SimpleDateFormat dateFormat = new SimpleDateFormat(user
						.getDateFormat().getFormat());
				SimpleDateFormat timeFormat = new SimpleDateFormat(user
						.getTimeFormat().getFormat());
				try {
					event.setStart(dateFormat.parse(start));
				} catch (Exception e) {
					olisActionSupport.addFieldError("eventDataManager.start",
							"error.invalid", "start");
				}
				try {
					event.setEnd(dateFormat.parse(end));
				} catch (Exception e) {
					olisActionSupport.addFieldError("eventDataManager.end",
							"error.invalid", "end");
				}
				try {
					event.setStartTime(timeFormat.parse(startTime));
				} catch (Exception e) {
					olisActionSupport.addFieldError(
							"eventDataManager.startTime", "error.invalid",
							"startTime");
				}
				try {
					event.setEndTime(timeFormat.parse(endTime));
				} catch (Exception e) {
					olisActionSupport.addFieldError("eventDataManager.endTime",
							"error.invalid", "endTime");
				}
			}
		} else {
			event.setId(eventId);
			event.setEventsBoard(new EventsBoard());
			event.setOwner(olisActionSupport.getUser());
		}
		return event;
	}

	public void setAllDay(Boolean allDay) {
		this.allDay = allDay;
	}

	public void setCity(City city) {
		this.city = city;
	}

	public void setCountries(List<Country> countries) {
		this.countries = countries;
	}

	public void setCountry(Country country) {
		this.country = country;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public void setEditable(Boolean editable) {
		this.editable = editable;
	}

	public void setEnd(String end) {
		this.end = end;
	}

	public void setEndTime(String endTime) {
		this.endTime = endTime;
	}

	public void setEventId(Long eventId) {
		this.eventId = eventId;
	}

	public void setFrequency(String frequency) {
		this.frequency = frequency;
	}

	public void setLocation(String location) {
		this.location = location;
	}

	public void setMonth(Integer month) {
		this.month = month;
	}

	public void setOlisActionSupport(OlisActionSupport olisActionSupport) {
		this.olisActionSupport = olisActionSupport;
	}

	public void setStart(String start) {
		this.start = start;
	}

	public void setStartTime(String startTime) {
		this.startTime = startTime;
	}

	public void setSubject(String subject) {
		this.subject = subject;
	}

	public void setYear(Integer year) {
		this.year = year;
	}

}
