/*
 * Created on May 19, 2008
 */
package br.pucrio.olis.web.calendar;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.springframework.dao.DataAccessException;

import br.pucrio.olis.business.BusinessException;
import br.pucrio.olis.business.service.CalendarService;
import br.pucrio.olis.domain.CalendarEvent;
import br.pucrio.olis.domain.CalendarUserService;
import br.pucrio.olis.domain.Frequency;
import br.pucrio.olis.util.OlisUtils;
import br.pucrio.olis.web.Constants;
import br.pucrio.olis.web.OlisActionSupport;

import com.opensymphony.xwork2.Preparable;

/**
 * <p>
 * This class is the action that displays the user calendar.
 * </p>
 * 
 * @author Ingrid Nunes
 * @version 1.0
 * 
 */
public class CalendarAction extends OlisActionSupport implements Preparable {

	/**
	 * <p>
	 * UID generated.
	 * </p>
	 */
	private static final long serialVersionUID = -1141791371308525266L;

	private CalendarService calendarService;
	private List<String> dayLabels;
	private List<List<CalendarDay>> days;
	private Integer month;
	private String monthLabel;
	private Integer year;

	/**
	 * <p>
	 * Create an instance of CalendarAction.
	 * </p>
	 * 
	 * @param calendarService
	 *            the calendar service.
	 */

	public CalendarAction(CalendarService calendarService) {
		this.calendarService = calendarService;
	}

	/**
	 * <p>
	 * Open the user calendar in a certain month.
	 * </p>
	 * 
	 * @return The "outcome" result code
	 * @throws Exception
	 *             on any error
	 */
	public String execute() throws Exception {
		try {
			Calendar calendar = GregorianCalendar.getInstance();
			calendar.set(Calendar.DAY_OF_MONTH, 1);
			calendar.set(Calendar.MONTH, this.month);
			calendar.set(Calendar.YEAR, this.year);
			calendar.set(Calendar.HOUR_OF_DAY, 0);
			calendar.set(Calendar.MINUTE, 0);
			calendar.set(Calendar.SECOND, 0);
			calendar.set(Calendar.MILLISECOND, 0);

			this.monthLabel = getText("month." + this.month);

			CalendarUserService calendarUserService = (CalendarUserService) getUser()
					.getService(CalendarUserService.class);
			Set<Integer> weekDays = Frequency.DAYS.get(calendarUserService
					.getShowWeekends() ? Frequency.DAILY
					: Frequency.EVERY_WEEKDAY);

			this.days = new ArrayList<List<CalendarDay>>(5);
			List<CalendarDay> currentWeek = new ArrayList<CalendarDay>(weekDays
					.size());

			if (calendarUserService.getShowWeekends()) {
				for (int i = 0; i < (calendar.get(Calendar.DAY_OF_WEEK) - 1); i++) {
					currentWeek.add(new CalendarDay());
				}
			} else {
				while (!weekDays.contains(calendar.get(Calendar.DAY_OF_WEEK))) {
					calendar.add(Calendar.DAY_OF_MONTH, 1);
				}
				for (int i = 1; i < (calendar.get(Calendar.DAY_OF_WEEK) - 1); i++) {
					currentWeek.add(new CalendarDay());
				}
			}

			Map<Frequency, List<CalendarEvent>> map = calendarService
					.loadByMonth(month, year, getUser());

			while (calendar.get(Calendar.MONTH) == this.month) {
				int dayOfMonth = calendar.get(Calendar.DAY_OF_MONTH);
				Integer weekDay = calendar.get(Calendar.DAY_OF_WEEK);

				if (weekDays.contains(weekDay)) {
					CalendarDay calendarDay = new CalendarDay(dayOfMonth);
					currentWeek.add(calendarDay);

					for (Frequency frequency : map.keySet()) {
						List<CalendarEvent> events = map.get(frequency);
						if (events != null) {
							if (Frequency.ONCE.equals(frequency)) {
								for (CalendarEvent calendarEvent : events) {
									Calendar start = GregorianCalendar
											.getInstance();
									start.setTime(calendarEvent.getEvent()
											.getStart());
									start.set(Calendar.HOUR_OF_DAY, 0);
									start.set(Calendar.MINUTE, 0);
									Calendar end = GregorianCalendar
											.getInstance();
									end.setTime(calendarEvent.getEvent()
											.getEnd());
									end.set(Calendar.HOUR_OF_DAY, 23);
									end.set(Calendar.MINUTE, 59);
									if (start.getTime().compareTo(
											calendar.getTime()) <= 0
											&& end.getTime().compareTo(
													calendar.getTime()) >= 0) {
										calendarDay.addEvent(calendarEvent);
									}
								}
							} else {
								for (CalendarEvent calendarEvent : events) {
									Set<Integer> days = Frequency.DAYS
											.get(frequency);
									if (days.contains(weekDay)) {
										if (calendarEvent.getEvent().getStart()
												.compareTo(calendar.getTime()) <= 0
												&& calendarEvent
														.getEvent()
														.getEnd()
														.compareTo(
																calendar
																		.getTime()) >= 0) {
											calendarDay.addEvent(calendarEvent);
										}
									}
								}
							}

						}
					}
				}

				if (calendar.get(Calendar.DAY_OF_WEEK) == Calendar.SATURDAY) {
					this.days.add(currentWeek);
					currentWeek = new ArrayList<CalendarDay>(weekDays.size());
				}

				calendar.add(Calendar.DAY_OF_MONTH, 1);
			}

			if (!currentWeek.isEmpty()) {
				while (currentWeek.size() < weekDays.size()) {
					currentWeek.add(new CalendarDay());
				}
				this.days.add(currentWeek);
			}
		} catch (BusinessException exc) {
			log.error(Constants.LOG_BUSINESS_ERROR);
			exc.printStackTrace();
			addActionError(exc);
		} catch (DataAccessException exc) {
			log.error(Constants.LOG_DATABASE_ERROR);
			exc.printStackTrace();
			addActionError(getText("error.unexcepected", exc.getMessage()));
		}

		return SUCCESS;
	}

	public List<String> getDayLabels() {
		return dayLabels;
	}

	public List<List<CalendarDay>> getDays() {
		return days;
	}

	public Integer getMonth() {
		return month;
	}

	public String getMonthLabel() {
		return monthLabel;
	}

	public Integer getYear() {
		return year;
	}

	/**
	 * <p>
	 * Open the user calendar in the next month.
	 * </p>
	 * 
	 * @return
	 * @throws Exception
	 */
	public String nextMonth() throws Exception {
		Calendar calendar = OlisUtils.getNextMonth(month, year);
		this.month = calendar.get(Calendar.MONTH);
		this.year = calendar.get(Calendar.YEAR);
		return execute();
	}

	/**
	 * <p>
	 * Open the user calendar in the current month.
	 * </p>
	 * 
	 * @return
	 * @throws Exception
	 */
	public String open() throws Exception {
		Calendar calendar = GregorianCalendar.getInstance();
		this.month = calendar.get(Calendar.MONTH);
		this.year = calendar.get(Calendar.YEAR);
		return execute();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.opensymphony.xwork2.Preparable#prepare()
	 */
	@Override
	public void prepare() throws Exception {
		this.dayLabels = new ArrayList<String>();
		CalendarUserService calendarUserService = (CalendarUserService) getUser()
				.getService(CalendarUserService.class);
		if (calendarUserService.getShowWeekends()) {
			for (Integer day : Frequency.DAYS.get(Frequency.DAILY)) {
				this.dayLabels.add(getText("day.of.week." + day));
			}
		} else {
			for (Integer day : Frequency.DAYS.get(Frequency.EVERY_WEEKDAY)) {
				this.dayLabels.add(getText("day.of.week." + day));
			}
		}
	}

	/**
	 * <p>
	 * Open the user calendar in the previous month.
	 * </p>
	 * 
	 * @return
	 * @throws Exception
	 */
	public String previousMonth() throws Exception {
		Calendar calendar = OlisUtils.getPreviousMonth(month, year);
		this.month = calendar.get(Calendar.MONTH);
		this.year = calendar.get(Calendar.YEAR);
		return execute();
	}

	public void setDayLabels(List<String> dayLabels) {
		this.dayLabels = dayLabels;
	}

	public void setDays(List<List<CalendarDay>> days) {
		this.days = days;
	}

	public void setMonth(Integer month) {
		this.month = month;
	}

	public void setMonthLabel(String monthLabel) {
		this.monthLabel = monthLabel;
	}

	public void setYear(Integer year) {
		this.year = year;
	}

}
