/*
 * Created on May 19, 2008
 */
package br.pucrio.olis.web.calendar;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.List;

import org.springframework.dao.DataAccessException;

import br.pucrio.olis.business.BusinessException;
import br.pucrio.olis.business.service.CalendarService;
import br.pucrio.olis.business.service.UserService;
import br.pucrio.olis.domain.CalendarEvent;
import br.pucrio.olis.domain.Event;
import br.pucrio.olis.domain.User;
import br.pucrio.olis.web.Constants;
import br.pucrio.olis.web.EventDataManager;
import br.pucrio.olis.web.OlisActionSupport;

import com.opensymphony.xwork2.Preparable;

/**
 * <p>
 * This class is the action that performs the calendar events management from
 * the calendar service.
 * </p>
 * 
 * @author Ingrid Nunes
 * @version 1.0
 * 
 */
public class CalendarEventAction extends OlisActionSupport implements
		Preparable {

	/**
	 * <p>
	 * UID generated.
	 * </p>
	 */
	private static final long serialVersionUID = -4251155600518829360L;

	private Boolean available;
	private CalendarService calendarService;
	private EventDataManager eventDataManager;
	private Long id;
	private List<Long> participants;
	private List<User> users;
	private UserService userService;

	/**
	 * <p>
	 * Create an instance of CalendarEventAction.
	 * </p>
	 * 
	 * @param eventDataManager
	 *            the event data manager.
	 * @param calendarService
	 *            the calendar service.
	 * @param userService
	 *            the user service.
	 */
	public CalendarEventAction(EventDataManager eventDataManager,
			CalendarService calendarService, UserService userService) {
		this.eventDataManager = eventDataManager;
		eventDataManager.setOlisActionSupport(this);
		this.calendarService = calendarService;
		this.userService = userService;
	}

	/**
	 * <p>
	 * Retrieve the city list according to the selected country.
	 * </p>
	 * 
	 * @return The "Success" result for this mapping
	 * @throws Exception
	 *             on any error
	 */
	public String city() throws Exception {
		return INPUT;
	}

	/**
	 * <p>
	 * Delete operation.
	 * </p>
	 * 
	 * @return The "outcome" result code
	 * @throws Exception
	 *             on any error
	 */
	public String delete() throws Exception {
		setTask(Constants.DELETE);
		return execute();
	}

	/**
	 * <p>
	 * Insert or update a Calendar Event object to the persistent store.
	 * </p>
	 * 
	 * @return The "outcome" result code
	 * @throws Exception
	 *             on any error
	 */
	public String execute() throws Exception {
		CalendarEvent calendarEvent = new CalendarEvent();

		if (hasErrors()) {
			return INPUT;
		}

		try {
			calendarEvent.setId(id);
			calendarEvent.setAvailable(available);
			calendarEvent.setOwner(getUser());
			if (this.participants != null) {
				for (Long idUser : this.participants) {
					User u = new User();
					u.setId(idUser);
					calendarEvent.addParticipants(u);
				}
			}
			calendarEvent.setEvent(eventDataManager.readEvent());

			if (Constants.CREATE.equals(getTask())) {
				calendarEvent = calendarService.save(calendarEvent);
			} else if (Constants.EDIT.equals(getTask())) {
				calendarEvent = calendarService.update(calendarEvent);
			} else if (Constants.DELETE.equals(getTask())) {
				calendarService.delete(calendarEvent);
			}

			Calendar calendar = GregorianCalendar.getInstance();
			calendar.setTime(calendarEvent.getEvent().getStart());
			eventDataManager.setMonth(calendar.get(Calendar.MONTH));
			eventDataManager.setYear(calendar.get(Calendar.YEAR));
		} catch (BusinessException exc) {
			log.error(Constants.LOG_BUSINESS_ERROR);
			exc.printStackTrace();
			addActionError(exc);
		} catch (DataAccessException exc) {
			log.error(Constants.LOG_DATABASE_ERROR);
			exc.printStackTrace();
			addActionError(getText("error.unexcepected", exc.getMessage()));
		}

		if (hasErrors()) {
			return INPUT;
		}

		return SUCCESS;
	}

	public Boolean getAvailable() {
		return available;
	}

	public EventDataManager getEventDataManager() {
		return eventDataManager;
	}

	public Long getId() {
		return id;
	}

	public List<Long> getParticipants() {
		return participants;
	}

	public List<User> getUsers() {
		return users;
	}

	/**
	 * <p>
	 * Retrieve Event object to be the event of the new calendar event being
	 * created.
	 * </p>
	 * 
	 * @return The "Input" result for this mapping
	 * @throws Exception
	 *             on any error
	 */
	public String importEvent() throws Exception {
		setTask(Constants.CREATE);
		Event event = calendarService.getEvent(eventDataManager.getEventId());
		eventDataManager.initializeFields(event);

		return INPUT;
	}

	/**
	 * <p>
	 * Retrieve CalendarEvent object to edit or null if CalendarEvent does not
	 * exist.
	 * </p>
	 * 
	 * @return The "Input" result for this mapping
	 * @throws Exception
	 *             on any error
	 */
	public String input() throws Exception {
		if (this.id == null) {
			setTask(Constants.CREATE);
			eventDataManager.initializeFields(null);
		} else {
			setTask(Constants.EDIT);
			CalendarEvent calendarEvent = calendarService.getCalendarEvent(id);
			available = calendarEvent.getAvailable();
			this.participants = new ArrayList<Long>(calendarEvent
					.getParticipants().size());
			for (User participant : calendarEvent.getParticipants()) {
				this.participants.add(participant.getId());
			}
			eventDataManager.initializeFields(calendarEvent.getEvent());
		}

		return INPUT;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.opensymphony.xwork2.Preparable#prepare()
	 */
	@Override
	public void prepare() throws Exception {
		eventDataManager.prepare();
		this.users = userService.loadAll();
		this.users.remove(getUser());
	}

	/**
	 * <p>
	 * Insert or update.
	 * </p>
	 * 
	 * @return The "outcome" result code
	 * @throws Exception
	 *             on any error
	 */
	public String save() throws Exception {
		return execute();
	}

	public void setAvailable(Boolean available) {
		this.available = available;
	}

	public void setEventDataManager(EventDataManager eventDataManager) {
		this.eventDataManager = eventDataManager;
	}

	public void setId(Long id) {
		this.id = id;
	}

	public void setParticipants(List<Long> participants) {
		this.participants = participants;
	}

	public void setUsers(List<User> users) {
		this.users = users;
	}

}
