/*
 * Created on May 22, 2008
 */
package br.pucrio.olis.web.eventsannouncement;

import java.util.Calendar;
import java.util.GregorianCalendar;

import org.springframework.dao.DataAccessException;

import br.pucrio.olis.business.BusinessException;
import br.pucrio.olis.business.service.EventsAnnouncementService;
import br.pucrio.olis.domain.CalendarUserService;
import br.pucrio.olis.domain.Event;
import br.pucrio.olis.domain.User;
import br.pucrio.olis.web.Constants;
import br.pucrio.olis.web.EventDataManager;
import br.pucrio.olis.web.OlisActionSupport;

import com.opensymphony.xwork2.Preparable;

/**
 * <p>
 * This class is the action that performs the events management from the events
 * announcement service.
 * </p>
 * 
 * @author Ingrid Nunes
 * @version 1.0
 * 
 */
public class EventAction extends OlisActionSupport implements Preparable {

	/**
	 * <p>
	 * UID generated.
	 * </p>
	 */
	private static final long serialVersionUID = -5277831502644253354L;

	private Boolean canImport;
	private EventDataManager eventDataManager;
	private EventsAnnouncementService eventsAnnouncementService;

	/**
	 * <p>
	 * Create an instance of EventAction.
	 * </p>
	 * 
	 * @param eventDataManager
	 *            the event data manager.
	 * @param eventsAnnouncementService
	 *            the events announcement service.
	 * @param locationService
	 *            the location service.
	 */
	public EventAction(EventDataManager eventDataManager,
			EventsAnnouncementService eventsAnnouncementService) {
		this.eventDataManager = eventDataManager;
		eventDataManager.setOlisActionSupport(this);
		this.eventsAnnouncementService = eventsAnnouncementService;
	}

	/**
	 * <p>
	 * Retrieve the city list according to the selected country.
	 * </p>
	 * 
	 * @return The "Success" result for this mapping
	 * @throws Exception
	 *             on any error
	 */
	public String city() throws Exception {
		return INPUT;
	}

	/**
	 * <p>
	 * Delete operation.
	 * </p>
	 * 
	 * @return The "outcome" result code
	 * @throws Exception
	 *             on any error
	 */
	public String delete() throws Exception {
		setTask(Constants.DELETE);
		return execute();
	}

	/**
	 * <p>
	 * Insert or update a Event object to the persistent store.
	 * </p>
	 * 
	 * @return The "outcome" result code
	 * @throws Exception
	 *             on any error
	 */
	public String execute() throws Exception {
		Event event = eventDataManager.readEvent();

		if (hasErrors()) {
			return INPUT;
		}

		try {
			if (Constants.CREATE.equals(getTask())) {
				event = eventsAnnouncementService.save(event);
			} else if (Constants.EDIT.equals(getTask())) {
				event = eventsAnnouncementService.update(event);
			} else if (Constants.DELETE.equals(getTask())) {
				eventsAnnouncementService.delete(event);
			}

			Calendar calendar = GregorianCalendar.getInstance();
			calendar.setTime(event.getStart());
			eventDataManager.setMonth(calendar.get(Calendar.MONTH));
			eventDataManager.setYear(calendar.get(Calendar.YEAR));
		} catch (BusinessException exc) {
			log.error(Constants.LOG_BUSINESS_ERROR);
			exc.printStackTrace();
			addActionError(exc);
		} catch (DataAccessException exc) {
			log.error(Constants.LOG_DATABASE_ERROR);
			exc.printStackTrace();
			addActionError(getText("error.unexcepected", exc.getMessage()));
		}

		if (hasErrors()) {
			return INPUT;
		}

		return SUCCESS;
	}

	public Boolean getCanImport() {
		return canImport;
	}

	public EventDataManager getEventDataManager() {
		return eventDataManager;
	}

	/**
	 * <p>
	 * Retrieve Event object to edit or null if Event does not exist.
	 * </p>
	 * 
	 * @return The "Input" result for this mapping
	 * @throws Exception
	 *             on any error
	 */
	public String input() throws Exception {
		if (eventDataManager.getEventId() == null) {
			setTask(Constants.CREATE);
			eventDataManager.initializeFields(null);
		} else {
			setTask(Constants.EDIT);
			Event event = eventsAnnouncementService.getEvent(eventDataManager
					.getEventId());
			eventDataManager.initializeFields(event);
		}
		this.canImport = canImport && eventDataManager.getEventId() != null;

		return INPUT;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.opensymphony.xwork2.Preparable#prepare()
	 */
	@Override
	public void prepare() throws Exception {
		eventDataManager.prepare();
		User user = getUser();
		this.canImport = new Boolean(user != null
				&& user.getService(CalendarUserService.class) != null);
	}

	/**
	 * <p>
	 * Insert or update.
	 * </p>
	 * 
	 * @return The "outcome" result code
	 * @throws Exception
	 *             on any error
	 */
	public String save() throws Exception {
		return execute();
	}

	public void setEventDataManager(EventDataManager eventDataManager) {
		this.eventDataManager = eventDataManager;
	}

}
