/*
 * Created on May 25, 2008
 */
package br.pucrio.olis.web.support;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.springframework.dao.DataAccessException;

import br.pucrio.olis.business.BusinessException;
import br.pucrio.olis.business.service.AcademicEventService;
import br.pucrio.olis.domain.AcademicEventType;
import br.pucrio.olis.domain.AcademicPreferences;
import br.pucrio.olis.domain.AreaOfInterest;
import br.pucrio.olis.domain.User;
import br.pucrio.olis.web.Constants;
import br.pucrio.olis.web.OlisActionSupport;

import com.opensymphony.xwork2.Preparable;

/**
 * <p>
 * This class is the action that performs the update of the academic user
 * preferences.
 * </p>
 * 
 * @author Ingrid Nunes
 * @version 1.0
 * 
 */
public class AcademicPreferencesAction extends OlisActionSupport implements
		Preparable {

	/**
	 * <p>
	 * UID generated.
	 * </p>
	 */
	private static final long serialVersionUID = -6861627383754134775L;

	private AcademicEventService academicEventService;
	private List<AreaOfInterest> areaOfInterestList;
	private List<Long> areasOfInterest;
	private List<String> eventTypes;
	private Boolean foreignEvents;
	private Long id;

	/**
	 * <p>
	 * Create an instance of AcademicPreferencesAction.
	 * </p>
	 * 
	 * @param academicEventService
	 *            the academic event service.
	 */
	public AcademicPreferencesAction(AcademicEventService academicEventService) {
		this.academicEventService = academicEventService;
	}

	/**
	 * <p>
	 * Update user preferences in the persistent store.
	 * </p>
	 * 
	 * @return The "outcome" result code
	 * @throws Exception
	 *             on any error
	 */
	public String execute() throws Exception {
		try {
			AcademicPreferences academicPreferences = new AcademicPreferences();
			academicPreferences.setId(id);
			academicPreferences.setUser(getUser());
			if (this.areasOfInterest != null) {
				for (Long id : this.areasOfInterest) {
					AreaOfInterest a = new AreaOfInterest();
					a.setId(id);
					academicPreferences.addAreasOfInterest(a);
				}
			}
			if (this.eventTypes != null) {
				for (String eventType : this.eventTypes) {
					academicPreferences.addEventTypes(AcademicEventType
							.valueOf(eventType));
				}
			}
			academicPreferences.setForeignEvents(foreignEvents);
			User user = academicEventService
					.updatePreferences(academicPreferences);
			setUser(user);
		} catch (BusinessException exc) {
			log.error(Constants.LOG_BUSINESS_ERROR);
			exc.printStackTrace();
			addActionError(exc);
		} catch (DataAccessException exc) {
			log.error(Constants.LOG_DATABASE_ERROR);
			exc.printStackTrace();
			addActionError(getText("error.unexcepected", exc.getMessage()));
		}

		if (hasErrors()) {
			return INPUT;
		}

		return SUCCESS;
	}

	public List<AreaOfInterest> getAreaOfInterestList() {
		return areaOfInterestList;
	}

	public List<Long> getAreasOfInterest() {
		return areasOfInterest;
	}

	public List<AcademicEventType> getEventTypeList() {
		return Arrays.asList(AcademicEventType.values());
	}

	public List<String> getEventTypes() {
		return eventTypes;
	}

	public Boolean getForeignEvents() {
		return foreignEvents;
	}

	public Long getId() {
		return id;
	}

	/**
	 * <p>
	 * Retrieve user preferences and populate the action fields.
	 * </p>
	 * 
	 * @return The "Success" result for this mapping
	 * @throws Exception
	 *             on any error
	 */
	public String input() throws Exception {
		User user = getUser();
		AcademicPreferences academicPreferences = (AcademicPreferences) user
				.getPreferences();
		this.id = academicPreferences.getId();
		this.foreignEvents = academicPreferences.getForeignEvents();
		this.areasOfInterest = new ArrayList<Long>(academicPreferences
				.getAreasOfInterest().size());
		for (AreaOfInterest areasOfInterest : academicPreferences
				.getAreasOfInterest()) {
			this.areasOfInterest.add(areasOfInterest.getId());
		}
		this.eventTypes = new ArrayList<String>(academicPreferences
				.getEventTypes().size());
		for (AcademicEventType eventType : academicPreferences.getEventTypes()) {
			this.eventTypes.add(eventType.toString());
		}
		return INPUT;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.opensymphony.xwork2.Preparable#prepare()
	 */
	@Override
	public void prepare() throws Exception {
		this.areaOfInterestList = academicEventService.loadAllAreasOfInterest();
	}

	public void setAreaOfInterestList(List<AreaOfInterest> areaOfInterestList) {
		this.areaOfInterestList = areaOfInterestList;
	}

	public void setAreasOfInterest(List<Long> areasOfInterest) {
		this.areasOfInterest = areasOfInterest;
	}

	public void setEventTypes(List<String> eventTypes) {
		this.eventTypes = eventTypes;
	}

	public void setForeignEvents(Boolean foreignEvents) {
		this.foreignEvents = foreignEvents;
	}

	public void setId(Long id) {
		this.id = id;
	}

}
