/*
 * Created on May 3, 2008
 */
package br.pucrio.olis.web.support;

import java.util.ArrayList;
import java.util.List;

import org.springframework.dao.DataAccessException;

import br.pucrio.olis.business.BusinessException;
import br.pucrio.olis.business.service.LocationService;
import br.pucrio.olis.business.service.UserService;
import br.pucrio.olis.domain.City;
import br.pucrio.olis.domain.Country;
import br.pucrio.olis.domain.User;
import br.pucrio.olis.web.Constants;
import br.pucrio.olis.web.OlisActionSupport;

import com.opensymphony.xwork2.Preparable;

/**
 * <p>
 * This class is the OlisActionSupport that supports the user management.
 * Through this class, the user can create, update or delete an user account.
 * </p>
 * 
 * @author Ingrid Nunes
 * @version 1.0
 * 
 */
public class UserManagementAction extends OlisActionSupport implements
		Preparable {

	/**
	 * <p>
	 * UID generated.
	 * </p>
	 */
	private static final long serialVersionUID = -2749102463214995321L;

	private List<Country> countries;
	private Country country;
	private LocationService locationService;
	private User user;
	private UserService userService;

	public UserManagementAction(UserService userService,
			LocationService locationService) {
		this.userService = userService;
		this.locationService = locationService;
	}

	/**
	 * <p>
	 * Retrieve the city list according to the selected country.
	 * </p>
	 * 
	 * @return The "Success" result for this mapping
	 * @throws Exception
	 *             on any error
	 */
	public String city() throws Exception {
		return INPUT;
	}

	/**
	 * <p>
	 * Insert or update an User object to the persistent store.
	 * </p>
	 * <p/>
	 * <p>
	 * If a User is not logged in, then a new User is created and automatically
	 * logged in. Otherwise, the existing User is updated.
	 * </p>
	 * 
	 * @return The "outcome" result code
	 * @throws Exception
	 *             on any error
	 */
	public String execute() throws Exception {
		boolean creating = Constants.CREATE.equals(getTask());
		creating = creating && isCreating(); // trust but verify

		User user = null;
		try {
			if (creating) {
				user = userService.save(this.user);
			} else {
				user = userService.update(this.user);
			}
		} catch (BusinessException exc) {
			log.error(Constants.LOG_BUSINESS_ERROR);
			exc.printStackTrace();
			addActionError(exc);
		} catch (DataAccessException exc) {
			log.error(Constants.LOG_DATABASE_ERROR);
			exc.printStackTrace();
			addActionError(getText("error.unexcepected", exc.getMessage()));
		}

		if (user != null) {
			super.setUser(user);
		}

		if (hasErrors()) {
			return INPUT;
		}

		return SUCCESS;
	}

	public List<City> getCities() throws BusinessException {
		if (country == null) {
			return new ArrayList<City>();
		} else {
			return locationService.loadCitiesByCountry(country);
		}
	}

	public List<Country> getCountries() {
		return countries;
	}

	public Country getCountry() {
		return country;
	}

	public User getUser() {
		return user;
	}

	/**
	 * <p>
	 * Retrieve User object to edit or null if User does not exist.
	 * </p>
	 * 
	 * @return The "Success" result for this mapping
	 * @throws Exception
	 *             on any error
	 */
	public String input() throws Exception {
		if (isCreating()) {
			this.user = new User();
			setTask(Constants.CREATE);
		} else {
			this.user = new User();
			setTask(Constants.EDIT);
			User currentUser = super.getUser();
			user.setId(currentUser.getId());
			user.setUsername(currentUser.getUsername());
			user.setPassword(currentUser.getPassword());
			user.setFirstName(currentUser.getFirstName());
			user.setLastName(currentUser.getLastName());
			user.setEmail(currentUser.getEmail());
			user.setDateOfBirth(currentUser.getDateOfBirth());
			user.setCity(currentUser.getCity());
			country = currentUser.getCity().getCountry();
		}

		return INPUT;
	}

	/**
	 * <p>
	 * Double check that there is not a valid User login.
	 * </p>
	 * 
	 * @return True if there is not a valid User login
	 */
	private boolean isCreating() {
		return (null == super.getUser());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.opensymphony.xwork2.Preparable#prepare()
	 */
	@Override
	public void prepare() throws Exception {
		this.countries = locationService.loadAllCountries();
	}

	/**
	 * <p>
	 * Insert or update an User.
	 * </p>
	 * 
	 * @return The "outcome" result code
	 * @throws Exception
	 *             on any error
	 */
	public String save() throws Exception {
		return execute();
	}

	public void setCountries(List<Country> countries) {
		this.countries = countries;
	}

	public void setCountry(Country country) {
		this.country = country;
	}

	public void setUser(User user) {
		this.user = user;
	}

}
