/*
 * Created on 23/06/2008
 */
package br.pucrio.olis.web.weather;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.springframework.dao.DataAccessException;

import br.pucrio.olis.business.BusinessException;
import br.pucrio.olis.business.service.LocationService;
import br.pucrio.olis.business.service.WeatherService;
import br.pucrio.olis.domain.City;
import br.pucrio.olis.domain.Country;
import br.pucrio.olis.domain.DegreeUnit;
import br.pucrio.olis.domain.User;
import br.pucrio.olis.domain.WeatherUserService;
import br.pucrio.olis.web.Constants;
import br.pucrio.olis.web.OlisActionSupport;

/**
 * <p>
 * This class is the action that performs the update of the weather service
 * options.
 * </p>
 * 
 * @author Ingrid Nunes
 * @version 1.0
 * 
 */
public class WeatherOptionsAction extends OlisActionSupport {

	/**
	 * <p>
	 * UID generated.
	 * </p>
	 */
	private static final long serialVersionUID = -218432179215465387L;

	private Country country;
	private String degreeUnit;
	private LocationService locationService;
	private WeatherService weatherService;
	private WeatherUserService weatherUserService;

	/**
	 * <p>
	 * Create an instance of WeatherOptionsAction.
	 * </p>
	 * 
	 * @param eventsAnnouncementService
	 *            the events announcement service.
	 */
	public WeatherOptionsAction(WeatherService weatherService,
			LocationService locationService) {
		this.weatherService = weatherService;
		this.locationService = locationService;
	}

	/**
	 * <p>
	 * Retrieve the city list according to the selected country.
	 * </p>
	 * 
	 * @return The "Success" result for this mapping
	 * @throws Exception
	 *             on any error
	 */
	public String city() throws Exception {
		return INPUT;
	}

	/**
	 * <p>
	 * Update events announcement service options in the persistent store.
	 * </p>
	 * 
	 * @return The "outcome" result code
	 * @throws Exception
	 *             on any error
	 */
	public String execute() throws Exception {
		try {
			weatherUserService.setDegreeUnit(DegreeUnit.valueOf(degreeUnit));
			User user = weatherService.update(weatherUserService);
			setUser(user);
		} catch (BusinessException exc) {
			log.error(Constants.LOG_BUSINESS_ERROR);
			exc.printStackTrace();
			addActionError(exc);
		} catch (DataAccessException exc) {
			log.error(Constants.LOG_DATABASE_ERROR);
			exc.printStackTrace();
			addActionError(getText("error.unexcepected", exc.getMessage()));
		}

		if (hasErrors()) {
			return INPUT;
		}

		return SUCCESS;
	}

	public List<City> getCities() throws BusinessException {
		if (country == null) {
			return new ArrayList<City>();
		} else {
			return locationService.loadCitiesByCountry(country);
		}
	}

	public List<Country> getCountries() throws BusinessException {
		return locationService.loadAllCountries();
	}

	public Country getCountry() {
		return country;
	}

	public String getDegreeUnit() {
		return degreeUnit;
	}

	public List<DegreeUnit> getDegreeUnits() throws BusinessException {
		return Arrays.asList(DegreeUnit.values());
	}

	public WeatherUserService getWeatherUserService() {
		return weatherUserService;
	}

	/**
	 * <p>
	 * Retrieve events announcement service options and populate the action
	 * fields.
	 * </p>
	 * 
	 * @return The "Success" result for this mapping
	 * @throws Exception
	 *             on any error
	 */
	public String input() throws Exception {
		User user = getUser();
		this.weatherUserService = (WeatherUserService) user
				.getService(WeatherUserService.class);
		this.country = weatherUserService.getDefaultCity().getCountry();
		this.degreeUnit = weatherUserService.getDegreeUnit().toString();
		return INPUT;
	}

	public void setCountry(Country country) {
		this.country = country;
	}

	public void setDegreeUnit(String degreeUnit) {
		this.degreeUnit = degreeUnit;
	}

	public void setWeatherUserService(WeatherUserService weatherUserService) {
		this.weatherUserService = weatherUserService;
	}

}
